## Copyright (C) 2018 jdonoghue
## 
## This program is free software: you can redistribute it and/or modify it
## under the terms of the GNU General Public License as published by
## the Free Software Foundation, either version 3 of the License, or
## (at your option) any later version.
## 
## This program is distributed in the hope that it will be useful, but
## WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
## 
## You should have received a copy of the GNU General Public License
## along with this program.  If not, see
## <https://www.gnu.org/licenses/>.

## -*- texinfo -*- 
## @deftypefn {} {@var{retval} =} arduino ()
## @deftypefnx {} {@var{retval} =} arduino (@var{port})
## @deftypefnx {} {@var{retval} =} arduino (@var{port}, @{board})
## @deftypefnx {} {@var{retval} =} arduino (@var{port}, @var{board}[, [@var{propname}, @var{propvalue}]*)
## Create a arduino object with a connection to an arduino board.
## 
## @var{port} - full patth of serial port to connect to. For linux, usually /dev/ttySXXX, for windows COMXX.
##
## @var{board} - name of board to connect (default is 'uno').
##
## @var{propname}, @var{propvalue} - property name and value pair for additional properties to pass to the creation
## of the arduino object.
## Currently properties are ignored.
##
## if the arduino function is called without parameters, it will scan for the first available arduino it can find and
## connect to it.
##
## Function returns a arduino object is successfully connected.
##
## @seealso{scanForArduinos}
## @end deftypefn 

classdef arduino < handle
  properties
    config = {};
    name = ""
    port = "<not set>";
    board = 'uno';
    debug = false;
    %debug = true;
    libraries = [];
    availablepins = [];
    connected = false;
  endproperties
  
methods
 
    function p = arduino (varargin)

      if (nargin == 0)
          arduinos = scanForArduinos (1);
	  if isempty (arduinos)
	    error ("arduino: No arduinos found on serial scan");
	  endif

          p.board = arduinos{1}.board;
	  p.config = eval (sprintf("config_%s", arduinos{1}.board));
          p.name = "arduino";
          p.port = arduinos{1}.port;
          p = __initArduino__(p);
      elseif (nargin == 1)
	arg0 = varargin{1};
        if (isa (arg0, "arduino"))
          # copy
          p.config = arg0.config;
          p.name = arg0.name;
          p.port = arg0.port;
          p.connected = arg0.connected;
          p.board = arg0.board;
        elseif ischar(arg0)
	  # port given
          p.config = config_uno;
          p.board = "uno";
          p.name = "arduino"
          p.port = arg0;
          p.connected = false;
          p = __initArduino__(p);
        else
          error ("arduino: port must be a string");         
        endif
      else
	printf("here %s\n", varargin{1});

        # at least port, board [optional property pairs]
	port = varargin{1};
	board = varargin{2};
        if !ischar(port)
          error ("arduino: port must be a string");         
	endif
        if !ischar(port)
          error ("arduino: board must be a string");         
	endif
	if mod(nargin, 2) != 0
          error ("arduino: expected property name, value pairs");
        endif
        if !iscellstr (varargin(3:2:nargin))
          error ("arduino: expected property names to be strings");
        endif
	p.port = port;
	p.name = ["arduino " board];
	p.board = board;
	p.config = eval(sprintf("config_%s", board));

	for i = 3:2:nargin
          propname = tolower(varargin{i});
          propvalue = varargin{i+1};

	  printf("%s = %s\n", propname, propvalue);
        endfor

        p = __initArduino__(p);
      endif
    endfunction

    function set_debug(p, d)
      p.debug = d;
    endfunction

    function d = get_debug(p)
      d = p.debug;
    endfunction

    function set_pin (p, pin, info)
      if ischar(pin)
        idx = find( cellfun(@(x) strcmpi(x.name, pin), p.config.pins), 1);
      else
        idx = find( cellfun(@(x) (x.d == pin), p.config.pins), 1);
      endif

      if isempty (idx)
        error ("arduino: unknown pin");
      endif
 
      p.config.pins{idx} = info;
    endfunction
      
    function info = get_pin (p, pin)
      if ischar(pin)
        idx = find( cellfun(@(x) strcmpi(x.name, pin), p.config.pins), 1);
      else
        idx = find( cellfun(@(x) (x.id == pin), p.config.pins), 1);
      endif

      if isempty (idx)
        error (["arduino: unknown pin " pin]);
      endif
     
      info = p.config.pins{idx};
    endfunction

    function retval = get_group(p,type)
      retval = {};

      for i = 1:numel(p.config.pins)
        pininfo = p.config.pins{i};
        idx = find( cellfun(@(x) strncmpi(x, type, length(type)), pininfo.modes), 1);
        if !isempty(idx)
          values = strsplit(pininfo.modes{idx}, "_");
          info = {};
          info.name = pininfo.name;
	  info.func = values{2};
	  info.mode = pininfo.mode;
	  info.owner = pininfo.owner;
	  retval{end+1}= info;
        endif
      endfor
    endfunction

    function retval = get_pingroup(p,pin,type)
      retval = {};
      pininfo = p.get_pin(pin);
      idx = find( cellfun(@(x) strncmpi(x, type, length(type)), pininfo.modes), 1);
      if !isempty(idx)
	# possibly this will be in format of
	# type[XX]_YY where XX is a number ir: spi0, spi1 etc, _YY will be the pinfunction ie: scl, miso etc
        values = strsplit(pininfo.modes{idx}, "_");
	type = values{1};

        for i = 1:numel(p.config.pins)
           pininfo = p.config.pins{i};
           idx = find( cellfun(@(x) strncmpi(x, type, length(type)), pininfo.modes), 1);
           if !isempty(idx)
             values = strsplit(pininfo.modes{idx}, "_");
             info = {};
	     info.name = pininfo.name;
	     info.func = values{2};
	     info.mode = pininfo.mode;
	     info.owner = pininfo.owner;
	     retval{end+1}= info;
           endif
	endfor
      endif
    endfunction

  endmethods 
endclassdef
